/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.accounts.Account;
import android.accounts.AccountManager;
import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.Preference.OnPreferenceClickListener;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.text.TextUtils;
import android.text.format.DateFormat;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.gtask.remote.GTaskSyncService;


public class NotesPreferenceActivity extends PreferenceActivity {
    //NotesPreferenceActivity，在小米便签中主要实现的是对背景颜色和字体大小的数据储存。
    // 继承了PreferenceActivity主要功能为对系统信息和配置进行自动保存的Activity
    public static final String PREFERENCE_NAME = "notes_preferences";
     //偏好项，首选项
    public static final String PREFERENCE_SYNC_ACCOUNT_NAME = "pref_key_account_name";
     //同步时间
    public static final String PREFERENCE_LAST_SYNC_TIME = "pref_last_sync_time";
     //设置案按键颜色
    public static final String PREFERENCE_SET_BG_COLOR_KEY = "pref_key_bg_random_appear";
    //
    private static final String PREFERENCE_SYNC_ACCOUNT_KEY = "pref_sync_account_key";
    //同步账户密码
    private static final String AUTHORITIES_FILTER_KEY = "authorities";
    //设置本地密码
    private PreferenceCategory mAccountCategory;
    //设置账户分组
    private GTaskReceiver mReceiver;
    //同步任务接收器
    private Account[] mOriAccounts;
    //账户
    private boolean mHasAddedAccount;
    // 账户的has标记
    @Override
    protected void onCreate(Bundle icicle) {//创建一个activity，在函数里要完成所有的正常静态设置
        super.onCreate(icicle);//.执行父类创建函数

        /* using the app icon for navigation */
        getActionBar().setDisplayHomeAsUpEnabled(true);//给左上角图标的左边加上一个返回的图标

        addPreferencesFromResource(R.xml.preferences);
        mAccountCategory = (PreferenceCategory) findPreference(PREFERENCE_SYNC_ACCOUNT_KEY);
        //根据同步账户密码进行账户分组
        mReceiver = new GTaskReceiver();//设置同步任务接收器
        IntentFilter filter = new IntentFilter();//设置过滤项
        filter.addAction(GTaskSyncService.GTASK_SERVICE_BROADCAST_NAME);
        registerReceiver(mReceiver, filter);
        //注册监听器，和对应的filter绑定
        mOriAccounts = null;
        View header = LayoutInflater.from(this).inflate(R.layout.settings_header, null);
        getListView().addHeaderView(header, null, true);
        //从xml获取Listview，获取listvivew，ListView的作用:用于列出所有选择
    }

    @Override
    protected void onResume() {//activity交互功能的实现，用于接受用户的输入
        super.onResume();//

        // need to set sync account automatically if user has added a new
        // account
        if (mHasAddedAccount) {//若用户新加了账户则自动设置同步账户
            Account[] accounts = getGoogleAccounts();//获取google同步账户
            if (mOriAccounts != null && accounts.length > mOriAccounts.length) {//若账户不为空且账户增加
                for (Account accountNew : accounts) {//遍历账户
                    boolean found = false;//更新账户
                    for (Account accountOld : mOriAccounts) {//循环判断当前账户列表中的账户是否与新建账户名相同
                        if (TextUtils.equals(accountOld.name, accountNew.name)) {
                            found = true;
                            break;//.若是没有找到旧的账户，那么同步账号中就只添加新账户
                        }
                    }
                    if (!found) {
                        setSyncAccount(accountNew.name);
                        break;
                    }
                }
            }
        }

        refreshUI();
    }

    @Override
    protected void onDestroy() {//销毁Activity
        if (mReceiver != null) {
            unregisterReceiver(mReceiver);//执行销毁操作
        }
        super.onDestroy();
    }

    private void loadAccountPreference() {//设置账户信息
        mAccountCategory.removeAll();//移除所有分组

        Preference accountPref = new Preference(this);//默认账户为当前账户
        final String defaultAccount = getSyncAccountName(this);
        accountPref.setTitle(getString(R.string.preferences_account_title));//首选项的大小标题
        accountPref.setSummary(getString(R.string.preferences_account_summary));//与google task同步便签记录
        accountPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {//设置preference的的监听器
            public boolean onPreferenceClick(Preference preference) {//功能描述：响应偏好对应元素的点击
                //函数实现：判断是否处于同步模式和默认的数据，指向不同的操作
                if (!GTaskSyncService.isSyncing()) {//若不在同步状态
                    if (TextUtils.isEmpty(defaultAccount)) {
                        // the first time to set account
                        showSelectAccountAlertDialog();
                    } else {
                        // if the account has already been set, we need to promp
                        // user about the risk
                        showChangeAccountConfirmAlertDialog();//已有账户则显示确认对话框
                    }
                } else {//若处于同步状态的情况下
                    Toast.makeText(NotesPreferenceActivity.this,
                            R.string.preferences_toast_cannot_change_account, Toast.LENGTH_SHORT)
                            .show();
                }//不能切换处于同步的账号
                return true;
            }
        });

        mAccountCategory.addPreference(accountPref);//根据新建首选项编辑新的账户分组
    }

    private void loadSyncButton() {//.设置同步按键和最近同步时间
        Button syncButton = (Button) findViewById(R.id.preference_sync_button);
        TextView lastSyncTimeView = (TextView) findViewById(R.id.prefenerece_sync_status_textview);
        //获取同步按键和同步时间显示
        // set button state
        if (GTaskSyncService.isSyncing()) {//设置按钮状态
            if (GTaskSyncService.isSyncing()) {//若是在同步状态下
            syncButton.setText(getString(R.string.preferences_button_sync_cancel));//
            syncButton.setOnClickListener(new View.OnClickListener() {//设置监听器
                public void onClick(View v) {
                    GTaskSyncService.cancelSync(NotesPreferenceActivity.this);
                }
            });
        } else {
            syncButton.setText(getString(R.string.preferences_button_sync_immediately));//非同步状态下按键显示“立即同步”，设置相关监听器
            syncButton.setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {//点击行为
                    GTaskSyncService.startSync(NotesPreferenceActivity.this);
                }//开始操作
            });
        }
        syncButton.setEnabled(!TextUtils.isEmpty(getSyncAccountName(this)));
        //如果没有账户，则不可选“立即同步”的按键
        // set last sync time
        if (GTaskSyncService.isSyncing()) {
            lastSyncTimeView.setText(GTaskSyncService.getProgressString());
            lastSyncTimeView.setVisibility(View.VISIBLE);//根据当前同步服务器设置时间显示框的文本以及可见性
        } else {//设置显示内容
            long lastSyncTime = getLastSyncTime(this);
            if (lastSyncTime != 0) {
                lastSyncTimeView.setText(getString(R.string.preferences_last_sync_time,
                        DateFormat.format(getString(R.string.preferences_last_sync_time_format),
                                lastSyncTime)));
                lastSyncTimeView.setVisibility(View.VISIBLE);
            } else {
                lastSyncTimeView.setVisibility(View.GONE);
            }//最近同步时间为空设置同步时间不可见
        }
    }

    private void refreshUI() {
        loadAccountPreference();
        loadSyncButton();
    }//刷新标签界面

    private void showSelectAccountAlertDialog() {//显示账户选择的对话框并进行账户的设置
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
        //创建一个新函数对话框
        View titleView = LayoutInflater.from(this).inflate(R.layout.account_dialog_title, null);
        //文本视图设置
        TextView titleTextView = (TextView) titleView.findViewById(R.id.account_dialog_title);
        titleTextView.setText(getString(R.string.preferences_dialog_select_account_title));
        TextView subtitleTextView = (TextView) titleView.findViewById(R.id.account_dialog_subtitle);
        subtitleTextView.setText(getString(R.string.preferences_dialog_select_account_tips));

        dialogBuilder.setCustomTitle(titleView);//设置标题及子标题内容
        dialogBuilder.setPositiveButton(null, null);//取消设置键按钮

        Account[] accounts = getGoogleAccounts();//获得谷歌当前账户列表
        String defAccount = getSyncAccountName(this);//默认账户

        mOriAccounts = accounts;
        mHasAddedAccount = false;//获得账户同步信息

        if (accounts.length > 0) {//如果有账户，则显示所有账户名称
            CharSequence[] items = new CharSequence[accounts.length];
            final CharSequence[] itemMapping = items;
            int checkedItem = -1;
            int index = 0;
            for (Account account : accounts) {//for循环检查账户列表
                if (TextUtils.equals(account.name, defAccount)) {
                    checkedItem = index;//查询目标账户
                }
                items[index++] = account.name;
            }
            dialogBuilder.setSingleChoiceItems(items, checkedItem,
                    new DialogInterface.OnClickListener() {//在对话框建立一个复选的对话框
                        public void onClick(DialogInterface dialog, int which) {//添加点击监听器
                            setSyncAccount(itemMapping[which].toString());//同步账户设置
                            dialog.dismiss();//取消对话框
                            refreshUI();//刷新界面
                        }
                    });
        }

        View addAccountView = LayoutInflater.from(this).inflate(R.layout.add_account_text, null);
        dialogBuilder.setView(addAccountView);
        //添加新账户，并为账户添加对话框视图
        final AlertDialog dialog = dialogBuilder.show();//显示对话框
        addAccountView.setOnClickListener(new View.OnClickListener() {//响应点击添加账户请求
            public void onClick(View v) {
                mHasAddedAccount = true;
                Intent intent = new Intent("android.settings.ADD_ACCOUNT_SETTINGS");
                intent.putExtra(AUTHORITIES_FILTER_KEY, new String[] {
                    "gmail-ls"
                });//建立网络组件
                startActivityForResult(intent, -1);//返回上一选项
                dialog.dismiss();
            }
        });
    }

    private void showChangeAccountConfirmAlertDialog() {//刷新标签界面
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
        //创建一个新的对话框
        View titleView = LayoutInflater.from(this).inflate(R.layout.account_dialog_title, null);
        TextView titleTextView = (TextView) titleView.findViewById(R.id.account_dialog_title);
        titleTextView.setText(getString(R.string.preferences_dialog_change_account_title,
                getSyncAccountName(this)));
        TextView subtitleTextView = (TextView) titleView.findViewById(R.id.account_dialog_subtitle);
        subtitleTextView.setText(getString(R.string.preferences_dialog_change_account_warn_msg));
        dialogBuilder.setCustomTitle(titleView);

        CharSequence[] menuItemArray = new CharSequence[] {
                getString(R.string.preferences_menu_change_account),
                getString(R.string.preferences_menu_remove_account),
                getString(R.string.preferences_menu_cancel)
        };//设置对话框自定义标题，对话框 可供选择的三种模式：改变同步账户，删除同步账户，取消
        dialogBuilder.setItems(menuItemArray, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                if (which == 0) {//若改变账户，则可显示该账户信息
                    showSelectAccountAlertDialog();
                } else if (which == 1) {
                    removeSyncAccount();
                    refreshUI();
                }
            }
        });//
        dialogBuilder.show();//显示对话框
    }

    private Account[] getGoogleAccounts() {
        AccountManager accountManager = AccountManager.get(this);
        return accountManager.getAccountsByType("com.google");
    }// 函数：获取谷歌账户，可通过账户管理器直接获取

    private void setSyncAccount(String account) {//设置同步账户
        if (!getSyncAccountName(this).equals(account)) {//若该账户不在同步列表内
            SharedPreferences settings = getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
            SharedPreferences.Editor editor = settings.edit();
            if (account != null) {
                editor.putString(PREFERENCE_SYNC_ACCOUNT_NAME, account);//若账户不为空，编辑账户的首选项
            } else {
                editor.putString(PREFERENCE_SYNC_ACCOUNT_NAME, "");
            }
            editor.commit();//提交数据

            // clean up last sync time
            setLastSyncTime(this, 0);

            // clean up local gtask related info
            new Thread(new Runnable() {
                public void run() {//清除本地的gtask关联的信息 将一些参数设置为0或NULL
                    ContentValues values = new ContentValues();
                    values.put(NoteColumns.GTASK_ID, "");
                    values.put(NoteColumns.SYNC_ID, 0);
                    getContentResolver().update(Notes.CONTENT_NOTE_URI, values, null, null);
                }
            }).start();

            Toast.makeText(NotesPreferenceActivity.this,
                    getString(R.string.preferences_toast_success_set_accout, account),
                    Toast.LENGTH_SHORT).show();//将toast的文本信息置为“设置账户成功”并显示出来
        }
    }

    private void removeSyncAccount() {//删除同步账户
        SharedPreferences settings = getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = settings.edit();//存放数据，设置共享首选项
        if (settings.contains(PREFERENCE_SYNC_ACCOUNT_NAME)) {//若当前账户首选项存在数据，就删除
            editor.remove(PREFERENCE_SYNC_ACCOUNT_NAME);
        }
        if (settings.contains(PREFERENCE_LAST_SYNC_TIME)) {//删除当前账户存在时间
            editor.remove(PREFERENCE_LAST_SYNC_TIME);
        }
        editor.commit();

        // clean up local gtask related info
        new Thread(new Runnable() {//
            public void run() {//清除本地的gtask关联的信息，将一些参数设置为0或NULL
                ContentValues values = new ContentValues();
                values.put(NoteColumns.GTASK_ID, "");
                values.put(NoteColumns.SYNC_ID, 0);
                getContentResolver().update(Notes.CONTENT_NOTE_URI, values, null, null);
            }
        }).start();
    }

    public static String getSyncAccountName(Context context) {//通过共享的首选项里的信息直接获取
        SharedPreferences settings = context.getSharedPreferences(PREFERENCE_NAME,
                Context.MODE_PRIVATE);//获取同步账户名称
        return settings.getString(PREFERENCE_SYNC_ACCOUNT_NAME, "");
    }

    public static void setLastSyncTime(Context context, long time) {//函数：设置最终同步的时间
        SharedPreferences settings = context.getSharedPreferences(PREFERENCE_NAME,
                Context.MODE_PRIVATE);//从共享首选项中找到相关账户并获取其编辑器
        SharedPreferences.Editor editor = settings.edit();
        editor.putLong(PREFERENCE_LAST_SYNC_TIME, time);
        editor.commit();
    }

    public static long getLastSyncTime(Context context) {////通过共享的首选项里的信息直接获取
        SharedPreferences settings = context.getSharedPreferences(PREFERENCE_NAME,
                Context.MODE_PRIVATE);//通过共享，获取时间
        return settings.getLong(PREFERENCE_LAST_SYNC_TIME, 0);
    }

    private class GTaskReceiver extends BroadcastReceiver {
        //响应收到的广播情况
        @Override
        public void onReceive(Context context, Intent intent) {//获取随广播而来的Intent中的同步服务的数据
            refreshUI();
            if (intent.getBooleanExtra(GTaskSyncService.GTASK_SERVICE_BROADCAST_IS_SYNCING, false)) {
                TextView syncStatus = (TextView) findViewById(R.id.prefenerece_sync_status_textview);
                syncStatus.setText(intent
                        .getStringExtra(GTaskSyncService.GTASK_SERVICE_BROADCAST_PROGRESS_MSG));
            }//如果在同步状态，改变同步按钮显示的文本

        }
    }

    public boolean onOptionsItemSelected(MenuItem item) {//处理菜单选项
        switch (item.getItemId()) {
            case android.R.id.home://返回主界面
                Intent intent = new Intent(this, NotesListActivity.class);
                //开始创建活动
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                startActivity(intent);//开始活动
                return true;
            default:
                return false;//若出现错误则返回
        }
    }
}
