package net.micode.notes.ui;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.text.method.HideReturnsTransformationMethod;
import android.text.method.PasswordTransformationMethod;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import android.widget.ToggleButton;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.model.WorkingNote;

public class PasswordLockActivity extends Activity {

    private static final String TAG ="PassewordLockActivity";

    private Button mbutton;

    private WorkingNote mWorkingNote;

    private String inputPwd;

    private final static int REQUEST_CODE_OPEN_NODE = 102;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        this.setContentView(R.layout.passwordlock_note);
        //the new 背景切换
        getWindow().setBackgroundDrawableResource(R.drawable.password_lock_background);

        long id = getIntent().getLongExtra(Intent.EXTRA_UID,0);

        mWorkingNote = WorkingNote.load(this,id);
        if (mWorkingNote == null) {
            Log.e(TAG, "load note failed with note id" + id);
            finish();
            return;
        }
        mbutton = (Button) findViewById(R.id.password_button_note);
        mbutton.setOnClickListener((l)->{
            testPassword(mWorkingNote.getNoteId());
        });
        //mWorkingNote = WorkingNote.load(this,);

    }

    private void openNote(long id) {
        Intent intent = new Intent(this, NoteEditActivity.class);
        intent.setAction(Intent.ACTION_VIEW);
        intent.putExtra("password",inputPwd);
        intent.putExtra(Intent.EXTRA_UID, id);
        this.startActivityForResult(intent, REQUEST_CODE_OPEN_NODE);
    }

    public void testPassword(long id){

        final AlertDialog.Builder builder = new AlertDialog.Builder(this);//初始化对话框
        View view = LayoutInflater.from(this).inflate(R.layout.dialog_pwd, null);//加载对话框的布局文件
        final EditText etPwd = (EditText) view.findViewById(R.id.set_private_pwd);//初始化对话框里的文本对象
        final ToggleButton passwordVisibleToggleButton = (ToggleButton) view.findViewById(R.id.password_visible_toggle_button);
        // 添加切换按钮的点击事件监听器
        passwordVisibleToggleButton.setOnClickListener(v -> {
            if (passwordVisibleToggleButton.isChecked()) {
                // 如果切换按钮被选中，显示密码文本
                etPwd.setTransformationMethod(HideReturnsTransformationMethod.getInstance());
            } else {
                // 否则隐藏密码文本
                etPwd.setTransformationMethod(PasswordTransformationMethod.getInstance());
            }
        });

        showSoftInput();//显示软键盘
        builder.setPositiveButton(android.R.string.ok, null);//确定按钮
        builder.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {//取消按钮
                hideSoftInput(etPwd);//隐藏软键盘
            }
        });
        final Dialog dialog = builder.setView(view).show();//显示出来

        final Button positive = (Button) dialog.findViewById(android.R.id.button1);//加载确定按钮布局文件
        positive.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {//设置确定按钮的接收器
                hideSoftInput(etPwd);//点击确定后收起软键盘
                String Password = etPwd.getText().toString();//文本框里的输入值
                if (!TextUtils.isEmpty(Password)) {//输入不为空
                    //Toast.makeText(getApplicationContext(), Password, Toast.LENGTH_SHORT).show();
                    inputPwd = Password;
                    openNote(id);
                }
                dialog.dismiss();//撤销对话框

            }

        });
        if (TextUtils.isEmpty(etPwd.getText())) {//如果文件夹名称为空，则不能选择确定按钮
            positive.setEnabled(false);
        }

        etPwd.addTextChangedListener(new TextWatcher() {//添加文本更改监听器,即判断文本更改状态
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {//文本更改之前（未完成）
                // TODO Auto-generated method stub
            }

            public void onTextChanged(CharSequence s, int start, int before, int count) {//当前文本改变触发，
                if (TextUtils.isEmpty(etPwd.getText())) {//文本为空确定按键不可用，不为空则可用
                    positive.setEnabled(false);
                } else {
                    positive.setEnabled(true);
                }
            }

            public void afterTextChanged(Editable s) {//文本修改之后（未完成）
                // TODO Auto-generated method stub
            }
        });
    }

    public void onBackPressed() {
        super.onBackPressed();
        //返回到NotesListActivity
        Intent intent = new Intent(this, NotesListActivity.class);
        startActivity(intent);
    }

    private void hideSoftInput(View view) {
        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        if (inputMethodManager != null) {
            inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(), 0);
        }
    }

    private void showSoftInput() {
        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        if (inputMethodManager != null) {
            inputMethodManager.toggleSoftInput(InputMethodManager.SHOW_FORCED, 0);
        }
    }


}
